#!/usr/bin/env python

"""add or remove file from repos file"""

import configparser
import os
from os.path import expanduser
import subprocess

from .common import Bcolors, query_yes_no
Bcolors = Bcolors()

def repo(repos, yes=None, backup=None, listall=None):
    """add or remove file from repos"""
    #config here
    home = expanduser('~')
    bunker_config = os.path.join(home, ".config-bunker.ini")
    config_exists = False
    if os.path.exists(bunker_config):
        config = configparser.ConfigParser()
        config.read(bunker_config)
        config_exists = True
        prefix = config['default']['prefix']
        repo_file = config['default']['repos']
    else:
        print(Bcolors.PALEYELLOW+"no config file found."+Bcolors.ENDC)
        print("please run: "+Bcolors.PINK+"bunker init"+Bcolors.ENDC)
        print()
        exit()
    # read repo file into array
    if listall:
        # list all repos
        print(Bcolors.GREY+"=== repositories ==="+Bcolors.ENDC)
        with open(prefix+repo_file, 'r') as fin:
            print(fin.read())
        exit()
    if not listall and not repos:
        print(Bcolors.FAIL+"ERROR: "+Bcolors.WARNING+"not enough arguments"+Bcolors.ENDC)
        print()
        exit()
    if os.path.exists(prefix+repo_file):
        with open(prefix+repo_file) as f:
            repofile_arr = f.readlines()
        nrf = []
        # strip chars and remove trailing /
        for x in repofile_arr:
            x = x.strip()
            if x[-1] == "/":
                x = x[:-1]
            nrf.append(x)
        repofile_arr = nrf
    else:
        repofile_arr = []
    newr = []
    for r in repos:
        if os.path.exists(r) and os.path.isdir(r):
            if os.path.isabs(r):
                if r[-1] == "/":
                    r = r[:-1]
                newr.append(r)
            else:
                r = os.path.abspath(r)
                if r[-1] == "/":
                    r = r[:-1]
                newr.append(r)
        else:
            print(Bcolors.WARNING+r+Bcolors.ORANGE+" does not exist, or isn't a directory."+Bcolors.ENDC)
            print(Bcolors.WARNING+"ignoring "+Bcolors.CYAN+r+Bcolors.ENDC)
    # loop sanitized list and add and remove items as needed
    for r in newr:
        if r in repofile_arr:
            if yes or query_yes_no("remove "+Bcolors.PALEYELLOW+r+Bcolors.ENDC+" from "+Bcolors.PALEBLUE+repo_file+"?"+Bcolors.ENDC, default="yes"):
                repofile_arr.remove(r)
                print(Bcolors.FAIL+r+Bcolors.ENDC)
        else:
            repofile_arr.append(r)
            print(Bcolors.OKGREEN+r+Bcolors.ENDC)
    # finally, backup and over-write editted array to repo file
    if backup:
        backup_repo = prefix+repo_file+".bak"
        print(Bcolors.PALEBLUE+"backing up repo file to: "+Bcolors.PALEYELLOW+backup_repo+Bcolors.ENDC)
        bcmd = "cp "+prefix+repo_file+" "+backup_repo
        subprocess.call(bcmd, shell=True)
    newrf = ""
    print("---")
    print(Bcolors.OKGREEN+"over-writing "+repo_file+Bcolors.ENDC)
    for r in repofile_arr:
        newrf += r+"\n"
    f = open(prefix+repo_file, "w+")
    f.write(newrf)
    f.close()

            

if __name__ == "__main__":
    repo()



#!/usr/bin/env python

"""source file in .profile. for bash prompts"""

import configparser
import os
from os.path import expanduser
import json
import time
import subprocess
import boto3

from .common import Bcolors, query_yes_no, countdown
Bcolors = Bcolors()

def prompt(instance_id=None, dostart=None, yes=None, run=None, bucket_info=None):
    """source file in .profile. for bash prompts"""
    #config here
    home = expanduser('~')
    bunker_config = os.path.join(home, ".config-bunker.ini")
    config = configparser.ConfigParser()
    config.read(bunker_config)

    prefix = config['default']['prefix']
    pem = prefix+config['default']['pem']
    if instance_id is None:
        instance_id = config['default']['instance_id']
    username = config['default']['username']
    client = boto3.client('ec2')
    ssm_client = boto3.client('ssm')
    if dostart:
        print(Bcolors.ENDC+"starting ec2 and waiting for "+Bcolors.OKGREEN+"\"running\""+Bcolors.ENDC+" state...")
        client.start_instances(InstanceIds=[instance_id])
        waiter = client.get_waiter('instance_running')
        waiter.wait(InstanceIds=[instance_id])
        countdown(20)
        print("instance up: "+str(instance_id))

    instance_ip = subprocess.run("aws ec2 describe-instances --instance-ids "+instance_id+" --query 'Reservations[*].Instances[*].PublicIpAddress' --output text", shell=True, stdout=subprocess.PIPE)
    instance_ip = instance_ip.stdout.decode('utf-8')
    instance_ip = instance_ip.strip(' \t\n\r')
    state = 0
    state = subprocess.run("aws ec2 describe-instances --instance-ids "+instance_id+" --query 'Reservations[*].Instances[*].State.Code' --output text", shell=True, stdout=subprocess.PIPE)
    state = state.stdout.decode('utf-8')
    try:
        state = int(state)
    except ValueError:
        print(Bcolors.RED+"instance does not seem to exist..."+Bcolors.ENDC)
        print("ensure everything is set correctly by running: "+Bcolors.PINK+"bunker init"+Bcolors.ENDC)
        print()
        exit()
    if state != 16:
        print(Bcolors.RED+"instance is not running."+Bcolors.ENDC)
        exit()
    if run is None:
        print("no script!")
        exit()
    else:
        if not os.path.isabs(run):
            print(Bcolors.PALEYELLOW+"Please use absolute paths to the location of the file on the EC2"+Bcolors.ENDC)
            print("example: "+Bcolors.PALEBLUE+"/home/ubuntu/.prompt.sh"+Bcolors.ENDC)
            exit()
    # INSTANCE READY FOR WORK, DO STUFF
    if yes or query_yes_no("install custom bash prompt with neofetch on: "+instance_ip+"?", "yes"):
        run_exploded = run.split("/")
        rel_run = run_exploded[-1]
        print(Bcolors.GOLD+bytes.decode(b'\xE2\x8F\xB3', 'utf8')+" sourcing "+rel_run+" in ~/.profile"+Bcolors.ENDC)
        if bucket_info != ['none', 'none', 'none'] and bucket_info is not None:
            print(Bcolors.CYAN+"will save output to: "+Bcolors.GOLD+bucket_info[0]+Bcolors.ENDC)
            cmd = "aws ssm send-command --instance-ids \""+instance_id+"\" --document-name \"AWS-RunShellScript\" --output-s3-bucket-name \""+bucket_info[0]+"\" --output-s3-region \""+bucket_info[1]+"\" --output-s3-key-prefix \""+bucket_info[2]+"\" --parameters commands=\"echo source "+run+" | sudo tee -a /home/ubuntu/.profile\""
        else:
            cmd = "aws ssm send-command --instance-ids \""+instance_id+"\" --document-name \"AWS-RunShellScript\" --parameters commands=\"echo source "+run+" | sudo tee -a /home/ubuntu/.profile\""
        ran_command = subprocess.check_output(cmd, shell=True).decode("utf-8").strip()
        ran_command = json.loads(ran_command)
        command_id = ran_command["Command"]["CommandId"]
        install_status = "InProgress"
        start_time = time.time()
        while install_status == "InProgress":
            elapsed_time = time.time() - start_time
            elapsed_time = time.strftime("%H:%M:%S", time.gmtime(elapsed_time))
            elapsed_time = str(elapsed_time)
            cmd = "aws ssm list-command-invocations --command-id \""+command_id+"\" --details" 
            ran_command = subprocess.check_output(cmd, shell=True).decode("utf-8").strip()
            ran_command = json.loads(ran_command)
            install_status = ran_command["CommandInvocations"][0]["Status"]
            if install_status == "InProgress":
                print(Bcolors.WARNING+install_status+Bcolors.GREY+" Time Elapsed: "+Bcolors.ENDC+elapsed_time+"     ", end="\r")
            else:
                if install_status != "Success":
                    color = Bcolors.FAIL
                else:
                    color = Bcolors.OKGREEN
                print(color+install_status+Bcolors.GREY+" Time Elapsed: "+Bcolors.ENDC+elapsed_time+"     ")
        # echo "neofetch" | sudo tee -a /home/ubuntu/.profile
        cmdoutput = ssm_client.get_command_invocation(
            CommandId=command_id,
            InstanceId=instance_id,
        )
        if cmdoutput['Status'] != "Success":
            time.sleep(1)
            print(Bcolors.GOLD+"running "+rel_run+" returned the following "+Bcolors.FAIL+"error:\n"+Bcolors.WARNING+cmdoutput['StandardErrorContent']+Bcolors.ENDC)

    complete = "complete"
    return complete

if __name__ == "__main__":
    prompt()



#!/usr/bin/env python

"""buildbunker uses boto3/ssm to clone all your git repos and then rsync up dot files and configs"""

import configparser
import os
from os.path import expanduser
import time
import subprocess
import boto3

from .common import Bcolors, query_yes_no, countdown, progbar
Bcolors = Bcolors()

def buildbunker(running=None, dostart=None, source=None, yes=None):
    """clone git repos on ec2, then copy over ignored files"""
    #config here
    home = expanduser('~')
    bunker_config = os.path.join(home, ".config-bunker.ini")
    config = configparser.ConfigParser()
    config.read(bunker_config)
    prefix = config['default']['prefix']
    repos = prefix+config['default']['repos']
    ignored = prefix+config['default']['ignored']
    pem = config['default']['pem']
    instance_id = config['default']['instance_id']
    username = config['default']['username']
    gitkeys = config['default']['gitkeys']
    gitkeys_arr = []
    keycount = gitkeys.count(', ')
    if keycount > 0:
        gitkeys_arr = config['default']['gitkeys'].split(", ")
    else:
        gitkeys_arr.append(config['default']['gitkeys'])
    # ec2 ssm client
    client = boto3.client('ec2')
    ssm_client = boto3.client('ssm')
    # start up if necessary
    if dostart:
        print(Bcolors.ENDC+"starting ec2 and waiting for \"running\" state...")
        client.start_instances(InstanceIds=[instance_id])
        waiter = client.get_waiter('instance_running')
        waiter.wait(InstanceIds=[instance_id])
        countdown(20)
        print("instance up: "+str(instance_id))
    # get ip address
    instance_ip = subprocess.run("aws ec2 describe-instances --instance-ids "+instance_id+" --query 'Reservations[*].Instances[*].PublicIpAddress' --output text", shell=True, stdout=subprocess.PIPE)
    instance_ip = instance_ip.stdout.decode('utf-8')
    instance_ip = instance_ip.strip(' \t\n\r')
    # get instance state
    state = 0
    state = subprocess.run("aws ec2 describe-instances --instance-ids "+instance_id+" --query 'Reservations[*].Instances[*].State.Code' --output text", shell=True, stdout=subprocess.PIPE)
    state = state.stdout.decode('utf-8')
    try:
        state = int(state)
    except ValueError:
        print(Bcolors.RED+"instance does not seem to exist..."+Bcolors.ENDC)
        print("ensure everything is set correctly by running: "+Bcolors.PINK+"bunker init"+Bcolors.ENDC)
        print()
        exit()
    state = int(state)
    if state != 16:
        print(Bcolors.RED+"instance is not running."+Bcolors.ENDC)
        if not running:
            stop = subprocess.run("aws ec2 stop-instances --instance-ids "+instance_id, shell=True, stdout=subprocess.PIPE)
        exit()
    # INSTANCE READY FOR WORK, DO STUFF
    #
    # auf gehts
    dirs = []
    diffs = []
    similarities = []
    if not os.path.isfile(repos) or os.stat(repos).st_size == 0:
        print(Bcolors.WARNING+"WARNING: "+Bcolors.GOLD+"you do not currently have any repos configured."+Bcolors.ENDC)
        print("it is recommended that you add a few repos to your repo file so bunker can determine a base path.")
        print("bunker removes this base path when it clones your repos on the EC2.")
        print("to add a repo to the repo file: "+Bcolors.PINK+"bunker repo path/to/repo"+Bcolors.ENDC)
        print(Bcolors.PALEBLUE+"NOTE: both relative and absolute paths will work."+Bcolors.ENDC)
    else:
        with open(repos, 'r') as f:
            x = 0
            for line in f:
                repo = line.strip()
                if len(repo) > 0:
                    if repo[-1] == "/":
                        repo = repo[:-1]
                    dirs.append(repo)
                    exploded_repo = repo.split("/")
                    for directory in exploded_repo:
                        if x == 0 and directory != "":
                            diffs.append(directory)
                        else:
                            if directory in diffs and directory not in similarities:
                                similarities.append(directory)
                x += 1
    if yes or query_yes_no("ready to transfer to "+instance_ip+"?", "yes"):
        addrepo = False
        if source:
            if not os.path.isdir(source):
                print(Bcolors.WARNING+"please use a valid directory"+Bcolors.ENDC)
                exit()
            if not os.path.isabs(source):
                source = os.path.abspath(source)
            if source[-1] == "/":
                source = source[:-1]
            if source not in dirs:
                addrepo = True
            dirs = []
            dirs.append(source)

        # check for known_hosts
        response = ssm_client.send_command(
            InstanceIds=[instance_id],
            DocumentName="AWS-RunShellScript",
            Parameters={'commands': ['ls /home/'+username+'/.ssh/known_hosts']}, )
        check_id = response['Command']['CommandId']
        time.sleep(1)
        checkoutput = ssm_client.get_command_invocation(
            CommandId=check_id,
            InstanceId=instance_id,
        )   
        while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
            time.sleep(1)
            checkoutput = ssm_client.get_command_invocation(
                CommandId=check_id,
                InstanceId=instance_id,
            )   
        if checkoutput['Status'] != "Success":
            print(Bcolors.WARNING+"no "+Bcolors.ORANGE+"known_hosts"+Bcolors.WARNING+" in /home/"+username+"/.ssh/"+Bcolors.ENDC)
            print("copy local known_hosts to EC2")
            if os.path.isfile(os.path.join(home, ".ssh", "known_hosts")):
                known_hosts = os.path.join(home, ".ssh", "known_hosts")
                #cmd = "scp -i "+pem+" "+known_hosts+" "+username+"@"+instance_ip+":/home/"+username+"/.ssh/"
                print("copy "+Bcolors.ORANGE+"~/.ssh/knownhosts"+Bcolors.ENDC+" to "+Bcolors.CYAN+instance_ip+Bcolors.ENDC)
                dest = "/home/"+username+"/.ssh/known_hosts"
                progbar(instance_ip, username, pem, known_hosts, dest, 0o644)
                #subprocess.call(cmd, shell=True)
            else:
                print(Bcolors.WARNING+"no known_hosts in ~/.ssh/"+Bcolors.ENDC)
                print("please configure your local ssh to connect to your git providers before using bunker.")
                print()
                exit()

        # check for config
        response = ssm_client.send_command(
            InstanceIds=[instance_id],
            DocumentName="AWS-RunShellScript",
            Parameters={'commands': ['ls /home/'+username+'/.ssh/config']}, )
        check_id = response['Command']['CommandId']
        time.sleep(1)
        checkoutput = ssm_client.get_command_invocation(
            CommandId=check_id,
            InstanceId=instance_id,
        )
        while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
            time.sleep(1)
            checkoutput = ssm_client.get_command_invocation(
                CommandId=check_id,
                InstanceId=instance_id,
            )
        if checkoutput['Status'] != "Success":
            print(Bcolors.WARNING+"no "+Bcolors.ORANGE+"config"+Bcolors.WARNING+" in /home/"+username+"/.ssh/"+Bcolors.ENDC)
            print("copy local config to EC2")
            if os.path.isfile(os.path.join(home, ".ssh", "config")):
                sshconfig = os.path.join(home, ".ssh", "config")
                #cmd = "scp -i "+pem+" "+sshconfig+" "+username+"@"+instance_ip+":/home/"+username+"/.ssh/"
                print("copy "+Bcolors.ORANGE+"~/.ssh/config"+Bcolors.ENDC+" to "+Bcolors.CYAN+instance_ip+Bcolors.ENDC)
                dest = "/home/"+username+"/.ssh/config"
                progbar(instance_ip, username, pem, sshconfig, dest, 0o644)
                #subprocess.call(cmd, shell=True)
            else:
                print(Bcolors.WARNING+"no config in ~/.ssh/"+Bcolors.ENDC)
                print("please configure your local ssh to connect to your git providers before using bunker.")
                print()
                exit()

        # check for .ssh/keys
        if gitkeys_arr[0] != "":
            for keyfile in gitkeys_arr:
                exploded = keyfile.split("/")
                kf = exploded[-1]
                response = ssm_client.send_command(
                    InstanceIds=[instance_id],
                    DocumentName="AWS-RunShellScript",
                    Parameters={'commands': ['ls /home/'+username+'/.ssh/'+kf]}, )
                check_id = response['Command']['CommandId']
                time.sleep(1)
                checkoutput = ssm_client.get_command_invocation(
                    CommandId=check_id,
                    InstanceId=instance_id,
                )
                while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
                    time.sleep(1)
                    checkoutput = ssm_client.get_command_invocation(
                        CommandId=check_id,
                        InstanceId=instance_id,
                    )
                if checkoutput['Status'] != "Success":
                    print(Bcolors.WARNING+"no "+Bcolors.ORANGE+kf+Bcolors.WARNING+" in /home/"+username+"/.ssh/"+Bcolors.ENDC)
                    print("copy local "+kf+" to EC2")
                    #cmd = "scp -i "+pem+" "+keyfile+" "+username+"@"+instance_ip+":/home/"+username+"/.ssh/"
                    print("copy "+Bcolors.ORANGE+"~/.ssh/"+kf+Bcolors.ENDC+" to "+Bcolors.CYAN+instance_ip+Bcolors.ENDC)
                    dest = "/home/"+username+"/.ssh/"+kf
                    progbar(instance_ip, username, pem, keyfile, dest, 0o600)
                    #subprocess.call(cmd, shell=True)

        # declare outputs dict
        outputs = {}
        for d in dirs:
            # loop thru repos (d), check if repo dir exists, then attempt to clone
            gitconfig = d+"/.git/config"
            catcher = False
            # getting repo url from .git/config
            with open(gitconfig, 'r') as gconfig:
                for line in gconfig:
                    if catcher:
                        repourl = line
                        break
                    if "[remote \"origin\"]" in line:
                        catcher = True
            repourl = repourl.strip(' \t\n\r')
            repoarr = repourl.split(" = ")
            repourl = repoarr[1]
            if "https://" in  repourl:
                repourl_arr = repourl.split("/")
                newrepourl = ""
                repo_trigger = False
                for part in repourl_arr:
                    if repo_trigger or ".com" in part or ".org" in part:
                        if "@" in part:
                            exp = part.split("@")
                            part = exp[1]
                        if ".com" in part or ".org" in part:
                            newrepourl += part+":"
                        else:
                            newrepourl += part+"/"
                        repo_trigger = True
                repourl = "git@"+newrepourl[:-1]

            exploded_d = d.split("/")
            newd = ""
            for thing in exploded_d:
                if thing not in similarities:
                    newd += thing+"/"
            if newd[0] != "/":
                newd = "/"+newd
            displayd = newd[1:]
            displayd_ex = displayd.split("/")
            displayd = displayd_ex[0]
            # asdf
            response = ssm_client.send_command(
                InstanceIds=[instance_id],
                DocumentName="AWS-RunShellScript",
                Parameters={'commands': ['ls /home/'+username+newd]}, )
            check_id = response['Command']['CommandId']
            # wait for response
            time.sleep(1)
            checkoutput = ssm_client.get_command_invocation(
                CommandId=check_id,
                InstanceId=instance_id,
            )
            while checkoutput['Status'] != "Success" and checkoutput['Status'] != "Failed":
                time.sleep(1)
                checkoutput = ssm_client.get_command_invocation(
                    CommandId=check_id,
                    InstanceId=instance_id,
                )
            if checkoutput['Status'] != "Success":
                response = ssm_client.send_command(
                    InstanceIds=[instance_id],
                    DocumentName="AWS-RunShellScript",
                    Parameters={'commands': ['sudo -u '+username+' git clone '+repourl+' /home/'+username+newd, 'chown -R '+username+':'+username+' /home/'+username+newd]}, )
                command_id = response['Command']['CommandId']
                outputs[command_id] = 'sudo -u '+username+' git clone '+repourl+' /home/'+username+newd, 'chown -R '+username+':'+username+' /home/'+username+newd
                print(Bcolors.YELLOW+"cloning repo: "+d+Bcolors.ENDC)
                time.sleep(1)
            else:
                print(Bcolors.OKGREEN+"repo exists: "+displayd+Bcolors.ENDC)
        if len(dirs) < 5:
            print(Bcolors.PALEYELLOW+"waiting for clone."+Bcolors.ENDC)
            countdown(10)
        # loop thru output and print any errors
        for key, value in outputs.items():
            cmdoutput = ssm_client.get_command_invocation(
                CommandId=key,
                InstanceId=instance_id,
            )
            if cmdoutput['Status'] != "Success":
                print(Bcolors.RED+"Warning - "+Bcolors.YELLOW+str(value)+Bcolors.ENDC)
                print(Bcolors.RED+"Command output ===\n"+Bcolors.YELLOW+cmdoutput['StandardErrorContent']+Bcolors.ENDC)
                print(Bcolors.RED+'===\nCloning Repo: '+cmdoutput['Status']+". "+Bcolors.YELLOW+"We will still attemp to rsync files"+Bcolors.ENDC)
        #
        #
        # backup hidden files and configs below
        #
        #
        print("cleaning up...")
        response = ssm_client.send_command(
            InstanceIds=[instance_id],
            DocumentName="AWS-RunShellScript",
            Parameters={'commands': ['chown -R '+username+':'+username+' /home/'+username]}, )
        command_id = response['Command']['CommandId']
        time.sleep(1)
        files = []
        with open(ignored, 'r') as f:
            for line in f:
                ignore = line.strip()
                files.append(ignore)
        print(Bcolors.PALEYELLOW+"copying over ignored files"+Bcolors.ENDC)
        for d in dirs:
            os.chdir(d)
            exploded_d = d.split("/")
            newd = ""
            for thing in exploded_d:
                if thing not in similarities:
                    newd += thing+"/"
            if newd[0] != "/":
                newd = "/"+newd
            displayd = newd[1:]
            displayd_ex = displayd.split("/")
            displayd = displayd_ex[0]
            print(Bcolors.BOLD+Bcolors.OKGREEN+"== "+displayd+" =="+Bcolors.ENDC)
            for i in files:
                exists = subprocess.run("find . -name '"+i+"'", shell=True, stdout=subprocess.PIPE)
                exists = exists.stdout.decode('utf-8')
                if len(exists) > 1:
                    print(Bcolors.BOLD+Bcolors.YELLOW+i+Bcolors.ENDC)
                    output = subprocess.check_output("find . -name \""+i+"\"", shell=True)
                    output = output.decode('utf-8')
                    output_arr = output.split("\n")
                    z = 1
                    for p in output_arr:
                        if z != len(output_arr):
                            abs_f = os.path.abspath(p)
                            mode = oct(os.stat(abs_f).st_mode & 0o777)
                            mode = int(mode, 8)
                            exploded_d = abs_f.split("/")
                            wo_sims = ""
                            for thing in exploded_d:
                                if thing not in similarities:
                                    wo_sims += thing+"/"
                            wo_sims = wo_sims[1:]
                            wo_sims = wo_sims[:-1]
                            print(Bcolors.PINK+wo_sims+Bcolors.ENDC)
                            dest = "/home/"+username+"/"+wo_sims
                            progbar(instance_ip, username, pem, abs_f, dest, mode)
                        z += 1
        if addrepo:
            if query_yes_no("would you like to add "+Bcolors.GOLD+dirs[0]+Bcolors.ENDC+" to your repos file?", "yes"): 
                with open(repos, "a") as myfile:
                    myfile.write(dirs[0]+"\n")
    else:
        if running:
            stop = subprocess.run("aws ec2 stop-instances --instance-ids "+instance_id, shell=True, stdout=subprocess.PIPE)
            print("\nshutting down")
        else:
            print("\nleaving ec2 running")
        exit()
    
    if running:
        stop = subprocess.run("aws ec2 stop-instances --instance-ids "+instance_id, shell=True, stdout=subprocess.PIPE)
        print(Bcolors.CYAN+"\nshutting down: "+instance_ip+Bcolors.ENDC)
    else:
        print(Bcolors.CYAN+"\nleaving "+instance_ip+" running"+Bcolors.ENDC)
    return

if __name__ == "__main__":
    buildbunker()



import sys
import time
import paramiko
from tqdm import tqdm

""" common functions """

def tqdmWrapViewBar(*args, **kwargs):
    pbar = tqdm(*args, **kwargs)  # make a progressbar
    last = [0]  # last known iteration, start at 0
    def viewBar2(a, b):
        pbar.total = int(b)
        pbar.update(int(a - last[0]))  # update pbar with increment
        last[0] = a  # update last known iteration
    return viewBar2, pbar  # return callback, tqdmInstance

def sftp_mkdir_p(sftp, remote_directory):
    dirs_exist = remote_directory.split('/')
    dirs_make = []
    # find level where dir doesn't exist
    while len(dirs_exist) > 0:
        try:
            sftp.listdir('/'.join(dirs_exist))
            break
        except IOError:
            value = dirs_exist.pop()
            if value == '':
                continue
            dirs_make.append(value)
        else:
            return False
    # ...and create dirs starting from that level
    for mdir in dirs_make[::-1]:
        dirs_exist.append(mdir)
        sftp.mkdir('/'.join(dirs_exist))

def progbar(ip_address, username, pem, source, destination, permissions=None):
    """upload a file to ec2 and display a progress bar."""
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    client.connect(ip_address, username=username, key_filename=pem)
    sftp = client.open_sftp()
    cbk, pbar = tqdmWrapViewBar(ascii=False, unit='b', unit_scale=True)
    dest_arr = destination.split("/")
    dest_wo_file = ""
    x = 1
    for part in dest_arr:
        if x != len(dest_arr):
            dest_wo_file += part+"/"
        x += 1
    sftp_mkdir_p(sftp, dest_wo_file)
    sftp.put(source,destination,callback=cbk)
    if permissions is not None:
        sftp.chmod(destination, permissions)
    pbar.close()
    client.close()

#progbar("54.91.190.120", "ubuntu", "/Users/neolaura/shindagger.pem", "/Users/neolaura/cubed-api/portalbearing/1.jpg", "/home/ubuntu/1.jpg")
def query_yes_no(question, default="yes"):
    '''confirm or decline'''
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)
    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '': 
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("\nPlease respond with 'yes' or 'no' (or 'y' or 'n').\n")

def countdown(num):
    while num > 0:
        fmtnum = str(num)
        if num < 10: 
            fmtnum = "0"+str(num)
        print(fmtnum, end="\r", flush=True)
        num -= 1
        time.sleep(1)
    return

class Bcolors:
    """console colors"""
    CYAN = '\033[96m'
    MAGENTA = '\033[95m'
    GREY = '\033[90m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    YELLOW = '\033[33m'
    RED = '\033[31m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    ORANGE = '\033[38;5;208m'
    PINK = '\033[38;5;212m'
    PALEYELLOW = '\033[38;5;228m'
    PALEBLUE = '\033[38;5;111m'
    GOLD = '\033[38;5;178m'
